/*
 * Copyright 2011 CodeGist.org
 *
 *     Licensed under the Apache License, Version 2.0 (the "License");
 *     you may not use this file except in compliance with the License.
 *     You may obtain a copy of the License at
 *
 *         http://www.apache.org/licenses/LICENSE-2.0
 *
 *     Unless required by applicable law or agreed to in writing, software
 *     distributed under the License is distributed on an "AS IS" BASIS,
 *     WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 *     See the License for the specific language governing permissions and
 *     limitations under the License.
 *
 *  ===================================================================
 *
 *  More information at http://www.codegist.org.
 */

package org.codegist.crest.serializer;

import org.codegist.common.io.IOs;
import org.codegist.crest.CRestConfig;

import java.io.InputStream;
import java.nio.charset.Charset;
import java.text.DateFormat;
import java.text.SimpleDateFormat;
import java.util.Date;

/**                                       
 * @author laurent.gilles@codegist.org
 */
public class DateDeserializer extends TypeDeserializer<Date> {

    private final DateFormat formatter;

    public DateDeserializer(CRestConfig crestConfig) {
        this.formatter = new SimpleDateFormat(crestConfig.getDateFormat());
    }

    /**
     * @inheritDoc
     */
    @Override
    protected Date deserialize(InputStream stream, Charset charset) throws Exception {
         synchronized (formatter) {
           return formatter.parse(IOs.toString(stream, charset, true));
         }
    }
}
